.BG
.FN heft.fit
.TL
Hazard Estimation with Flexible Tails.
.DN
Hazard estimation using splines to
approximate the log-hazard function.
.CS
heft.fit(data, delta, penalty, knots, leftlin=F, 
        shift, leftlog, rightlog, maxknots, silent=T)
.RA
.AG data
vector of observations. Observations may or may not be right censored. All
observations should non-negative.
.OA
.AG delta
binary vector with the same length as data. Elements of `data'
for which the corresponding element of `delta' is 0 are assumed to be
right censored, elements of `data'
for which the corresponding element of `delta' is 1 are assumed to be
uncensored. If `delta' is missing, all observations are assumed
to be uncensored.
.AG penalty
the parameter to be used in the AIC criterion. The method chooses
the number of knots that minimizes -2*loglikelihood+penalty*(number of knots).
The default is to use `penalty=log(samplesize)' as in BIC. The effect of
this parameter is summarized in `heft.summary'.
.AG knots
ordered vector of values, which forces the method to start with these knots.
If `knots' is not specified, a default knot-placement rule is employed.
.AG leftlin
if `leftlin' is `T' an extra basis-function, which is linear to the left
of the first knot, is included in the basis.
If any of the data is exactly 0, the default of `leftlin' is `T'.
.AG shift
parameter for the log terms. Default is `quantile(data[delta==1],.75)'.
.AG leftlog
coefficient of `log(x/(x+shift))', which must be greater than `-1'.
(In particular, if `leftlog = 0' no `log(x/(x+shift))' term
is included.) If `leftlog' is missing its maximum likelihood estimate is used.
If any of the data is exactly zero, `leftlog' is 
set to zero.
.AG rightlog
coefficient of `log(x+shift)', which must be greater than or 
equal to `-1'. (In particular, if `rightlog = 0' no 
`log(x+shift)' term is included.)
If `rightlog' is missing its maximum likelihood estimate is used.
.AG maxknots
maximum number of knots allowed in the model (default is `4*length(data)^0.2').
.AG silent
suppresses the printing of diagnostic output about knots added or deleted,
Rao-statistics, Wald-statistics and log-likelihoods.
.RT
The output is organized to serve as input for `heft.plot',
`heft.summary', `dheft', `hheft', `pheft', `qheft' and `rheft'.
The function returns a list with the following members:
.RC knots
vector of the locations of the knots in the logspline model.
.RC logl
the k-th element is the log-likelihood of the fit with k knots.
.RC thetak
coefficients of the knot part of the
spline. The k-th coefficient is the coefficient
of  (x-t(k))^3+. If a coefficient is zero the corresponding
knot was deleted from the model.
.RC thetap
coefficients of the polynomial part of the spline.
The first element is the constant term and
the second element is the linear term.
.RC thetal
coefficients of the logarithmic terms. The first element equals
`leftlog' and the second element equals `rightlog'.
.RC penalty
the penalty that was used.
.RC shift
parameter used in the definition of the log terms.
.RC sample
the sample size that was used.
.RC logse
the standard errors of `thetal'.
.RC max
the maximum element of `data'.
.RC ad
vector indicating whether a model of this dimension
was not fit (2), fit during the addition stage (0) or during
the deletion stage (1).
.EX
fit <- heft.fit(data)
.SH REFERENCE
Kooperberg, C, Stone, C. J. and Truong, Y. K. (1995) Hazard Regression,
.ul
Journal of the American Statistical Association,
\fB90\fP, 78-94.        
.SA
`heft.plot', `heft.summary', `dheft', `hheft', `pheft', `qheft', `rheft'.
.WR
